@@ -12,12 +12,12 @@ use Carp;
 use vars
     qw($VERSION @ISA @EXPORT @EXPORT_OK @EXPORT_FAIL %EXPORT_TAGS 
        %sizeof $INITIALIZED $RUN_TIME $OS_CACHE_DIR),
-    qw($FATAL_EXCEPTIONS $SAFE_EXCEPTIONS $REGRESS),           # exceptional
-    qw($CLIENT_NAME $CACHE_SIZE $TRANSACTION_PRIORITY),        # tied
-    qw($DEFAULT_OPEN_MODE),                                    # simulated
-    qw(%SCHEMA $EXCEPTION %CLASSLOAD $CLASSLOAD $CLASS_AUTO_LOAD);     # private
+    qw($SAFE_EXCEPTIONS $REGRESS @UNLOADED),                        # exceptional
+    qw($CLIENT_NAME $CACHE_SIZE $TRANSACTION_PRIORITY),             # tied
+    qw($DEFAULT_OPEN_MODE),                                         # simulated
+    qw(%SCHEMA $EXCEPTION %CLASSLOAD $CLASSLOAD $CLASS_AUTO_LOAD);  # private
 
-$VERSION = '1.43';
+$VERSION = '1.46';
 
 $OS_CACHE_DIR = $ENV{OS_CACHE_DIR} || '/tmp/ostore';
 if (!-d $OS_CACHE_DIR) {
@@ -60,7 +60,7 @@ require DynaLoader;
 
 $EXCEPTION = sub {
     my $m = shift;
-    local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+#    local $Carp::CarpLevel = $Carp::CarpLevel + 1;  # too ambitious
     if ($m eq 'SEGV') {
 	$m = ObjStore::Transaction::SEGV_reason();
 	if ($m) {
@@ -91,7 +91,7 @@ $EXCEPTION = sub {
 $SIG{SEGV} = \&$EXCEPTION
     unless defined $SIG{SEGV}; # MUST NOT BE CHANGED! XXX
 
-eval { require Thread::Specific; } or do {
+eval { require Thread::Specific } or do {
     sub lock {}
     undef $@;
 };
@@ -153,7 +153,7 @@ sub PoweredByOS {
     "$Config::Config{sitelib}/ObjStore/PoweredByOS.gif"; #give it a shot... XXX
 }
 
-$FATAL_EXCEPTIONS = 1;   #happy default for newbies... (or my co-workers :-)
+my $FATAL_EXCEPTIONS = 1;   #happy default for newbies... (or my co-workers :-)
 sub fatal_exceptions {
     my ($yes) = @_;
     if (!$FATAL_EXCEPTIONS and $yes) {
@@ -208,8 +208,7 @@ set_stargate(\&DEFAULT_STARGATE);
 sub bless ($;$) {
     my ($ref, $class) = @_;
     $class ||= scalar(caller);
-    my $old = blessed $ref;
-    $ref->BLESS($class) if $old;
+    $ref->BLESS($class) if blessed $ref;
     $class->BLESS($ref);
 }
 # When CORE::GLOBAL works -
@@ -247,6 +246,8 @@ sub force_load {
 #    warn "force_load $class $isa";
 
     $ {"${class}::UNLOADED"} = 1;
+    push @UNLOADED, $class;
+
     for (my $x=0; $x < @$isa; $x+=2) {
 	push @{"${class}::ISA"}, $isa->[$x];
 	force_load($isa->[$x], $isa->[$x+1]);
@@ -264,22 +265,38 @@ $CLASS_AUTO_LOAD = 1;
 sub _isa_loader {
     no strict 'refs';
     my ($bs, $base, $class) = @_;
-#    warn "_isa_loader $bs $base $class";
+#    Carp::cluck "_isa_loader $bs $base $class";
     if (!@{"$class\::ISA"} and $CLASS_AUTO_LOAD) {
 	return $class if $class eq 'ObjStore::Database';
-	my $isa = $bs ? $bs->[3] : [$base,[]];
-	($class,$isa) = @$isa
-	    if ($class =~ m/^_fake\:\:/ and @$isa == 2);
+	my $isa;
+	if (!$bs) {
+	    $isa = [$base,[]];
+	} else {
+	    $isa = $bs->[3];
+	}
+	if ($class =~ m/^_fake\:\:/ and @$isa == 2) {
+	    # pop fake blessing
+	    ($class,$isa) = @$isa;
+	}
 	require_isa_tree($class, $isa);
-	force_load($class, $isa);
+	if (!$class->isa($base)) {
+	    force_load($class, $isa);
+	    while (!$class->isa($base) and @$isa) {
+		# pop classes until we get a winner
+		($class,$isa) = @$isa;
+	    }
+	    if (!$class->isa($base)) {
+		# oops!  hope this works...
+		return $base;
+	    }
+	}
     }
+#    warn $class;
     $class;
 };
 $CLASSLOAD = \&_isa_loader;
 
-sub disable_auto_class_loading {
-    $CLASS_AUTO_LOAD = 0;
-}
+sub disable_auto_class_loading { $CLASS_AUTO_LOAD=0 }
 
 sub lookup {
     my ($path, $mode) = @_;
@@ -345,7 +362,7 @@ sub debug {
     if ($mask) {
 	Carp->import('verbose');
     }
-    $ObjStore::REGRESS = $mask != 0;
+    ++$ObjStore::REGRESS if $mask != 0;
     _debug($mask);
 }
 
@@ -583,6 +600,7 @@ sub _versionof {
 
 sub _is_evolved {
     my ($o, $txn) = @_;
+    warn "is_evolved might be depreciated";
     croak("is_evolved($o) is only meaningful on real objects") if !ref $o;
     my $x = sub {
 	my $bs = $o->_blessto_slot;
@@ -625,7 +643,7 @@ sub isa_versions {
 }
 
 sub _engineer_blessing {
-    my ($br, $bs, $o, $toclass) = @_;
+    my ($br, $bs, $o, $toclass, $os_class) = @_;
     if (! $bs) {
 	# This warning is broken since it doesn't detect the right thing
 	# when there are multiple databases.  Each database needs its own copy
@@ -633,9 +651,11 @@ sub _engineer_blessing {
 #	warn "ObjStore::BRAHMA must be notified of run-time manipulation of VERSION strings by changing \$ObjStore::RUN_TIME to be != \$CLASS_DOGTAG{$toclass}" 
 #	    if ($CLASS_DOGTAG{$toclass} or 0) == $ObjStore::RUN_TIME; #majify? XXX
 
-	$bs = $br->{$toclass} =
-	    [1, $toclass, $ObjStore::RUN_TIME,
-	     isa_tree($toclass,0), isa_versions($toclass, {}, 0)];
+	$bs = $br->{$toclass} = [1,
+				 $toclass,
+				 $ObjStore::RUN_TIME,
+				 isa_tree($toclass,0),
+				 isa_versions($toclass, {}, 0)];
 	$bs->const;
 	$CLASS_DOGTAG{$toclass} = $bs->[2];
 #	warn "fix $toclass ".$bs->[2];
@@ -645,6 +665,7 @@ sub _engineer_blessing {
 
 sub iscorrupt {
     my ($o, $vlev) = @_;
+    warn "iscorrupt might be depreciated";
     $vlev = 'all' if !defined $vlev;
     if ($vlev !~ m/^\d+$/) {
 	if ($vlev eq 'quiet') { $vlev = 0; }
@@ -752,12 +773,19 @@ sub new {
     ObjStore::bless($db, $class) if $db;
 }
 
+use vars qw(%BLESSMAP);
 sub import_blessing {
     my ($db) = @_;
     my $bs = $db->_blessto_slot;
     if ($bs) {
-	my $class = &ObjStore::_isa_loader($bs, 'ObjStore::Database', $bs->[1]);
+	# This is essentially the same as what is done for
+	# ObjStore::UNIVERSAL.
 
+	my $class = $BLESSMAP{ $bs->[1] };
+	if (!$class) {
+	    $class = $BLESSMAP{ $bs->[1] } =
+		&ObjStore::_isa_loader($bs, 'ObjStore::Database', $bs->[1]);
+	}
 	# Must use CORE::bless here -- the database is _already_ blessed, yes?
 	CORE::bless($db, $class);
     }
@@ -811,7 +839,7 @@ sub BLESS {
 
 	&ObjStore::begin('update', sub {
 		my $br = $db->_conjure_brahma;
-		_engineer_blessing($br, scalar(_get_certified_blessing($br, $db, $class)), $db, $class);
+		_engineer_blessing($br, scalar(_get_certified_blessing($br, $db, $class)), $db, $class, 'ObjStore::Database');
 	    });
 	die if $@;
     }
@@ -1048,7 +1076,9 @@ use vars qw($VERSION);
 $VERSION = '1.00';
 use Carp;
 use overload ('""' => \&_pstringify,
-	      'bool' => sub {1},
+	      'bool' => sub () {1},
+	      '0+' => \&_pnumify,
+	      '+' => \&_pnumify,
 	      '==' => \&_peq,
 	      '!=' => \&_pneq,
 #	      'nomethod' => sub { croak "overload: ".join(' ',@_); }
@@ -1073,7 +1103,7 @@ sub BLESS {
     if (_is_persistent($r) and !$ {"$class\::UNLOADED"}) {
 	# recode in XS ? XXX
 	my $br = $r->database_of->_conjure_brahma;
-	_engineer_blessing($br, scalar(_get_certified_blessing($br, $r, $class)), $r, $class);
+	_engineer_blessing($br, scalar(_get_certified_blessing($br, $r, $class)), $r, $class, $r->os_class());
     }
     $class->SUPER::BLESS($r);
 }
@@ -1095,8 +1125,7 @@ sub new_ref {
 	if !defined $seg;
     my $type;
     if (!defined $safe) {
-	carp "default reference type will change to unsafe soon";
-	$type = 0;
+	$type = 1;
     }
     elsif ($safe eq 'safe') { $type=0; }
     elsif ($safe eq 'unsafe' or $safe eq 'hard') { $type=1; }
@@ -1346,12 +1375,22 @@ use vars qw($VERSION @ISA %REP);
 $VERSION = '1.01';
 @ISA='ObjStore::AV';
 
-for (qw(configure add remove)) {
+for (qw(configure add remove index_path)) {
     ObjStore::_mark_method($_)
 }
 
 sub new { require ObjStore::REP; &ObjStore::REP::load_default }
 
+sub index_path {
+    # make generic? "forward to rep_class" XXX
+    my $o = $_[0];
+    my $rep = $o->rep_class;
+    my $m = $rep->can('index_path');
+    croak "$rep does not support 'index_path' yet"
+	if !$m;
+    $m->(@_);
+}
+
 #----------- ----------- ----------- ----------- ----------- -----------
 
 # only for indices keyed by a single string
